﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2016 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using System.Linq;
using HEAL.Attic;
using HeuristicLab.Common;
using HeuristicLab.Core;
using HeuristicLab.Data;
using HeuristicLab.Optimization;

namespace HeuristicLab.Encodings.BinaryVectorEncoding.SolutionModel {
  [Item("Univariate solution model (binary)", "")]
  [StorableType("E30B8452-970C-4AF2-B7E7-4AC9C21928BA")]
  public sealed class UnivariateModel : Item, ISolutionModel<BinaryVector> {
    [Storable]
    public DoubleArray Probabilities { get; set; }
    [Storable]
    public IRandom Random { get; set; }

    [StorableConstructor]
    private UnivariateModel(StorableConstructorFlag _) : base(_) { }
    private UnivariateModel(UnivariateModel original, Cloner cloner)
      : base(original, cloner) {
      Probabilities = cloner.Clone(original.Probabilities);
      Random = cloner.Clone(original.Random);
    }
    public UnivariateModel(IRandom random, int N) : this(random, Enumerable.Range(0, N).Select(x => 0.5).ToArray()) { }
    public UnivariateModel(IRandom random, double[] probabilities) {
      Probabilities = new DoubleArray(probabilities);
      Random = random;
    }
    public UnivariateModel(IRandom random, DoubleArray probabilties) {
      Probabilities = probabilties;
      Random = random;
    }

    public override IDeepCloneable Clone(Cloner cloner) {
      return new UnivariateModel(this, cloner);
    }

    public BinaryVector Sample() {
      var vec = new BinaryVector(Probabilities.Length);
      for (var i = 0; i < Probabilities.Length; i++)
        vec[i] = Random.NextDouble() < Probabilities[i];
      return vec;
    }
  }
}
